% Wrapper for ecoc that makes ecoc compatible with nfoldxval.
%
% Requires the SVM toolbox by Anton Schwaighofer.
%
% INPUTS
%   nin             - dimensionality of data
%   func_net2init   - binary classifier init (see nfoldxval)
%   net2params      - binary classifier parameters (see nfoldxval)
%   nclasses        - number of classes (currently only 3<=nclasses<=7 suppored)
%   use01targets    - see ecoc
%
% OUTPUTS
%   net     - see ecoc
%
% DATESTAMP
%   29-Sep-2005  2:00pm
%
% See also ECOC, NFOLDXVAL, CLF_ECOC_CODE

% Piotr's Image&Video Toolbox      Version 1.0   
% Written and maintained by Piotr Dollar    pdollar-at-cs.ucsd.edu 
% Please email me if you find bugs, or have suggestions or questions! 
 
function net = clf_ecoc(nin,func_net2init,net2params,nclasses,use01targets)
    if( nclasses<3 || nclasses>7 )
        error( 'currently only works if 3<=nclasses<=7'); end;
    if( nargin<5 || isempty(use01targets)) use01targets=0; end;
    
    % create code (limited for now)
    [C,nbits] = clf_ecoc_code( nclasses );
    net = ecoc(nclasses, nbits, C );
    net.verbosity = 0; % don't diplay output

    % initialize and temporarily store binary learner
    net.templearner = feval( func_net2init, nin, net2params{:} );

    % ecoctrain2 is custom version of ecoctrain
    net.fun_train = @clf_ecoctrain;
    net.fun_fwd = @ecocfwd;
    
    
function net = clf_ecoctrain( net, varargin )
    net = ecoctrain( net, net.templearner, varargin{:} );
