/*********************************************************************
* DATESTAMP  29-Sep-2005  2:00pm
* Piotr's Image&Video Toolbox      Version 1.0   
* Written and maintained by Piotr Dollar    pdollar-at-cs.ucsd.edu 
* Please email me if you find bugs, or have suggestions or questions! 
*********************************************************************/

#include <math.h>
#include "mex.h"

/*******************************************
 * dist_euclidean
 *
 * Slower then parallel matlab version by about a factor of 10!
 * Just here for comparison purposes.
 *
 *******************************************/

void dist_euclidean(double* D, const double* X, const double* Y, const int m, const int n, const int p)
{
    int i, j, k;

    for (j = 0; j < n; j++)
        for (i = 0; i < m; i++)
            D[j*m+i] = 0.0;
      
    for (k = 0; k < p; k++) {
        for (j = 0; j < n; j++) {
            for (i = 0; i < m; i++) {
                D[j*m+i] += (X[k*m+i] - Y[k*n+j])*(X[k*m+i] - Y[k*n+j]);
            }
        }
    }
}

/******************************************************************************/
void mexFunction (int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[])
{
  double *X, *Y, *D;
  int m, n, p;

  /* Check for proper number of arguments */
  if (nrhs != 2) 
      mexErrMsgTxt ("Two input arguments required.");
  else if (nlhs > 1)
      mexErrMsgTxt("Too many output arguments.");

  /* extract arguments */
  X = (double *) mxGetData(prhs[0]);
  Y = (double *) mxGetData(prhs[1]);  
  m = mxGetM(prhs[0]);
  n = mxGetM(prhs[1]);
  p = mxGetN(prhs[1]);

  /* create output array */  
  plhs[0] = mxCreateDoubleMatrix( m, n, mxREAL );
  D = (double*) mxGetData(plhs[0]);
  
  /* Do the actual computations in a subroutine */
  dist_euclidean(D,X,Y,m,n,p);
}
